from typing import Any, Dict, Optional, Union

from hvac.exceptions import VaultError
from pydantic import Field
from pydantic.main import BaseModel

from sitri.providers.contrib.json import JsonConfigProvider
from sitri.providers.contrib.vault import VaultKVConfigProvider
from sitri.settings.base import BaseLocalModeConfig, BaseLocalModeSettings


class VaultKVLocalProviderArgs(BaseModel):
    json_path: str = Field(...)
    default_path_mode_state: bool = Field(default=True)


class VaultKVSettings(BaseLocalModeSettings):
    @property
    def _local_provider_args(self):
        args = self.__config__.local_provider_args
        if args:
            if isinstance(args, dict):
                args = VaultKVLocalProviderArgs(**args)

        return args

    @property
    def local_provider(self) -> JsonConfigProvider:
        if not self.__config__.local_provider:
            args = self._local_provider_args

            if args:
                self.__config__.local_provider = JsonConfigProvider(
                    json_path=args.json_path,
                    default_path_mode_state=args.default_path_mode_state,
                    logger=self.__config__.local_provider_logger,
                )
            else:
                raise ValueError("Local provider arguments not found for local mode")

        return self.__config__.local_provider

    def _build_local(self):
        d: Dict[str, Optional[str]] = {}

        for field in self.__fields__.values():
            value: Optional[str] = None

            if self.__config__.local_mode_path_prefix:
                path = f"{self.__config__.local_mode_path_prefix}{self.local_provider.separator}{field.name}"

            else:
                path = field.name

            try:
                value = self.local_provider.get(key=path)
            except VaultError:
                self.local_provider.logger.error(f"Could not get local variable {path}")

            if field.is_complex():
                value = self._build_complex_value(value, path)

            if value is None and field.default is not None:
                value = field.default

            d[field.name] = value

        return d

    def _build_default(self):
        d: Dict[str, Optional[str]] = {}

        provider = self.__config__.provider

        for field in self.__fields__.values():
            vault_val: Optional[str] = None

            vault_secret_path = field.field_info.extra.get("vault_secret_path")
            vault_mount_point = field.field_info.extra.get("vault_mount_point")
            vault_secret_key = field.field_info.extra.get("vault_secret_key")

            if vault_secret_key is None:
                vault_secret_key = field.name

            try:
                vault_val = provider.get(
                    key=vault_secret_key,
                    secret_path=vault_secret_path if vault_secret_path else self.__config__.default_secret_path,
                    mount_point=vault_mount_point if vault_mount_point else self.__config__.default_mount_point,
                )
            except VaultError:
                provider.logger.warning(
                    f'Could not get secret "{vault_mount_point}/{vault_secret_path}:{vault_secret_key}"'
                )

            if field.is_complex():
                vault_val = self._build_complex_value(
                    vault_val, f"{vault_mount_point}/{vault_secret_path}:{vault_secret_key}"
                )

            if vault_val is None and field.default is not None:
                vault_val = field.default

            d[field.name] = vault_val

        return d

    class VaultKVSettingsConfig(BaseLocalModeConfig):
        provider: VaultKVConfigProvider
        default_secret_path: Optional[str] = None
        default_mount_point: Optional[str] = None

        local_mode: bool = False

        local_mode_path_prefix: Optional[str] = None
        local_provider_args: Optional[Union[VaultKVLocalProviderArgs, Dict]]

        local_provider: Optional[JsonConfigProvider] = None

        local_provider_logger: Optional[Any] = None

    __config__: VaultKVSettingsConfig



import logging
import typing


def get_default_logger(name: typing.Optional[str] = None) -> logging.Logger:
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s - [%(levelname)s] - %(name)s - (%(filename)s).%(funcName)s(%(lineno)d) - %(message)s",
    )

    return logging.getLogger(name or "sitri.default")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sitri',
 'sitri.providers',
 'sitri.providers.base',
 'sitri.providers.contrib',
 'sitri.providers.contrib.vault',
 'sitri.settings',
 'sitri.settings.contrib',
 'sitri.settings.contrib.vault',
 'sitri.strategy']

package_data = \
{'': ['*']}

extras_require = \
{'all': ['redis>=4.2.2,<5.0.0',
         'vedis>=0.7.1,<0.8.0',
         'pyyaml>=6.0,<7.0',
         'hvac>=0.11.2,<0.12.0',
         'pydantic==1.9.1',
         'structlog==21.5.0',
         'ujson>=5.2.0,<6.0.0'],
 'hvac': ['hvac>=0.11.2,<0.12.0'],
 'pyyaml': ['pyyaml>=6.0,<7.0'],
 'redis': ['redis>=4.2.2,<5.0.0'],
 'settings': ['hvac>=0.11.2,<0.12.0', 'pydantic==1.9.1'],
 'structlog': ['structlog==21.5.0'],
 'ujson': ['ujson>=5.2.0,<6.0.0'],
 'vedis': ['vedis>=0.7.1,<0.8.0']}

setup_kwargs = {
    'name': 'sitri',
    'version': '2.1.0',
    'description': 'Library for one endpoint config managment',
    'long_description': '<p align="center">\n  <a href="https://github.com/elastoo-team/sitri">\n    <img src="https://raw.githubusercontent.com/Elastoo-Team/sitri/master/docs/_static/full_logo.jpg">\n  </a>\n  <h1 align="center">\n    Sitri - powerful settings & configs for python\n  </h1>\n</p>\n\n[![PyPI](https://img.shields.io/pypi/v/sitri)](https://pypi.org/project/sitri/)\n[![codecov](https://codecov.io/gh/Elastoo-Team/sitri/branch/master/graph/badge.svg)](https://codecov.io/gh/elastoo-team/sitri)\n[![Maintainability](https://api.codeclimate.com/v1/badges/625f1d869adbf4128f75/maintainability)](https://codeclimate.com/github/Elastoo-Team/sitri/maintainability)\n![Code Climate technical debt](https://img.shields.io/codeclimate/tech-debt/Elastoo-Team/sitri)\n[![Build Status](https://img.shields.io/endpoint.svg?url=https%3A%2F%2Factions-badge.atrox.dev%2FElastoo-Team%2Fsitri%2Fbadge&style=popout)](https://actions-badge.atrox.dev/Elastoo-Team/sitri/goto)\n[![Read the Docs](https://img.shields.io/readthedocs/sitri)](https://sitri.readthedocs.io)\n\nSitri - library for managing authorization and configuration data from a single object with possibly different or identical providers\n\n#  Installation\n\n```bash\npoetry add sitri -E "all"\n```\n\nor\n```bash\npip3 install sitri[all]\n```\n\n*Extras* packs and providers:\n\n    1. all - all providers and settings module.\n    2. settings - pydantic, providers with settings-support.\n    3. redis - for redis provider.\n    4. hvac - for HashiCorp Vault provider.\n    5. vedis - for vedis provider.\n    6. pyyaml - for YAML provider.\n    7. structlog - structlog package if your app does not provide structlog (optional, default logging package also support).\n    8. ujson - speed-up your json provider (optional).\n\n\n# Basics with SystemProvider\n\n```python\nfrom sitri.providers.contrib import SystemConfigProvider\nfrom sitri import Sitri\n\nconf = Sitri(\n    config_provider=SystemConfigProvider(prefix="basics"),\n)\n```\nSystem provider use system environment for get config data. For unique - sitri lookup to "namespace" by prefix.\n\nExample:\n\n*In console:*\n```bash\nexport BASICS_NAME=Huey\n```\n\n*In code:*\n```python\nname = conf.get_config("name")\n\nprint(name)  # output: Huey\n```\n\n#  Docs\nRead base API references and other part documentation on https://sitri.readthedocs.io/\n',
    'author': 'Alexander Lavrov',
    'author_email': 'egnod@ya.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Egnod/sitri',
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
